/** BEGIN COPYRIGHT BLOCK
 * This Program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 * 
 * This Program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * In addition, as a special exception, Red Hat, Inc. gives You the additional
 * right to link the code of this Program with code not covered under the GNU
 * General Public License ("Non-GPL Code") and to distribute linked combinations
 * including the two, subject to the limitations in this paragraph. Non-GPL Code
 * permitted under this exception must only link to the code of this Program
 * through those well defined interfaces identified in the file named EXCEPTION
 * found in the source code files (the "Approved Interfaces"). The files of
 * Non-GPL Code may instantiate templates or use macros or inline functions from
 * the Approved Interfaces without causing the resulting work to be covered by
 * the GNU General Public License. Only Red Hat, Inc. may make changes or
 * additions to the list of Approved Interfaces. You must obey the GNU General
 * Public License in all respects for all of the Program code and other code used
 * in conjunction with the Program except the Non-GPL Code covered by this
 * exception. If you modify this file, you may extend this exception to your
 * version of the file, but you are not obligated to do so. If you do not wish to
 * provide this exception without modification, you must delete this exception
 * statement from your version and license this file solely under the GPL without
 * exception. 
 * 
 * 
 * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * END COPYRIGHT BLOCK **/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include "pio.h"
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

int iii_pio_procparse (
	const char *cmd,
	int count,
	struct iii_pio_parsetab *tb
)
{
  FILE *fp;
  char buf[8192];
  int rc = 0;

  fp = popen(cmd,"r");

  if (fp == NULL) {
    return -1;
  }
  
  while (fgets(buf,8192,fp) != NULL && rc >= 0) {
    char *rp;
    int i;

    rp = strchr(buf,'\n');

    if (rp) {
      *rp = '\0';
    }

    rp = strchr(buf,':');

#if defined(__osf__)
    if (rp == NULL) {
      rp = strchr(buf,'=');
    }
#endif

    if (rp == NULL) continue;
    
    *rp = '\0';
    rp++;
    while(isspace(*rp)) rp++;

    for (i = 0; i < count; i++) {
      if (strcmp(tb[i].token,buf) == 0) {
	rc = (tb[i].fn)(buf,rp);
	break;
      }
    }
  }
  
  pclose(fp);
  
  return rc;
}

int iii_pio_getnum (
	const char *cmd,
	long *valPtr
)
{
  FILE *fp;
  char buf[8192];
  
  fp = popen(cmd,"r");
  
  if (fp == NULL) {
    return -1;
  }
  
  if (fgets(buf,8192,fp) == NULL) {
    pclose(fp);
    return -1;
  }

  pclose(fp);

  if (!(isdigit(*buf))) {
    return -1;
  }
  
  *valPtr = atol(buf);
  
  return 0;
}
